//----------------------------------------------------------------------
// Titel     : DSP Heimradio mit ATTINY85 fuer SI4735 Radio
//----------------------------------------------------------------------
// Funktion  : Misst Spannung via Poti und ermittelt Stufen zur
//				 fixen Frequenzstufenwahl
// Schaltung : siehe B. Kainka
// Dateiname : DSP_heimradio1.c
// Includes  : i2cmaster.h 
// Hinweis   : i2cmaster.S (Assem. Source) den source Files hinzuf.
//----------------------------------------------------------------------
// Prozessor : Attiny85
// Takt       : 1,00 MHz (RC Oszillator 8MHz, CLKDIV8 aktiviert)
// Sprache   : C
// Datum     : 27.12.2013 22:05:33
// Version   : 1
// Autor     : Andreas Nestler
//----------------------------------------------------------------------
// Pinconfig:
// =====================
// PB0 -> SDA	 (O)	-> TWI Daten
// PB1 -> Taste  (I) PU	-> Taste an GND
// PB2 -> SCL	 (O)	-> TWI Takt  
// PB3 -> unused (I) 	-> auf GND legen,
// PB4 -> ADC2   (AI) 	-> Potischleifer
// PB5 -> unused (I) PU -> reserviert als RESET-Pin
//
// Legende: I = Input , O = Output, AI = Analog Input, PU = Pull-up
//
//----------------------------------------------------------------------

#ifndef F_CPU
#define F_CPU 1000000UL
#endif

#include <inttypes.h>      // Allgemeine Bibliotheken
#include <avr/io.h>
#include <avr/interrupt.h>
// #include <avr/signal.h> // obsolete replaced by interrupt.h
#include <util/delay.h>
#include <avr/eeprom.h>
#include "inc/i2cmaster.h"
// #include "inc/USI_UART.h" // add USI_UART.c to source files

/* Hardware IO abstraction macros */

/* Button abstraction */
#define BUTTON_PUSHED (!(PINB & (1<<PINB1))) // !, da Taster an PB1 nach GND
/* Bit Manupulation */
#define SETBIT(VAR, BIT) ((VAR) |= (1 << (BIT))) // set bit
#define CLEARBIT(VAR, BIT) ((VAR) &= (unsigned)~(1 << (BIT))) // clear bit
#define TOGGLEBIT(VAR, BIT) ((VAR) ^= (1 << (BIT))) // Toogle Bit
#define CHECKBIT(VAR, BIT) ((VAR) & (1 << (BIT))) // Check if Bit is set

/* Defines to avoid compiler warnings */
#define UNUSED(x) ((void)(x))
// Doppel-Cast, um compiler warning zu vermeiden:
// 'cast to pointer from integer of different size'
#define PUINTCAST(x) ((unsigned int *)(uintptr_t) (x))


/*
** constant definitions
*/
#define Dev4735  0x22   // device address of SI4735 see datasheet
						// -> 34d  = 0x11 once left shifted
#define F1 8890			// Freq.: N-Joy
#define F2 9350			// Freq.: NDR2
#define F3 9730			// Freq.: Antenne-MV
#define F4 10480		// Freq.: Ostseewelle
#define F5 10650		// Freq.: DLF

/*
** function prototypes
*/ 
void si4735init(void);
void si4735freq(unsigned int iFreq);
void Scan(void);
unsigned int Fm_tune_status(void);
unsigned int ADC_Read(unsigned char channel);
void ADC_Init(void);


int main(void)
{
    unsigned char cT, cN, cNmax, cCh=0, cCh_old;
	unsigned int iFreq, iPoti, iDiv, * pUint_EEaddr;
		
        
	DDRB = (1<<DDB0) | (1<<DDB2);	// Portpins B.0 und B.2 auf Ausgang,
    PORTB = (1<<PB0) | (1<<PB2);	// und diese Bits auf 1
	PORTB |= (1<<PB1) | (1<<PB5);	// noch Pull-up f. Inputs PB1 u. PB5 ein
	DIDR0 = (1<<DDB4);				// Disable dig. input buffer for PB4->ADC2
									// this saves current consumption
	
	ADC_Init();
	i2c_init();
	
	_delay_ms(300);
	si4735init();
	_delay_ms(800);

    if BUTTON_PUSHED {				// Programmier-Modus wenn Taste 
	  do {							// beim Einschalten gedrckt
	    _delay_ms(10);				// Tasten-Entprellung
	  } while (BUTTON_PUSHED);
	  iFreq=8700;
	  si4735freq(iFreq);			// Frequenz auf UKW-Bandanfang einstellen
	  _delay_ms(200);
	  pUint_EEaddr=0;				// setze Pointer auf 0 -> EEAddr 0
	  for (;;) { 					// program loop forever
	    Scan();						// Neuen kanal suchen
		do {
	      _delay_ms(10);
		} while (!BUTTON_PUSHED);
		cT=0;						// cT initialisieren
		do {
		  _delay_ms(10);
		  cT++;   									// Lnge des Tastenducks messen -> 10ms inkrement
		  if (cT > 200) cT=200;						// Limitieren auf 2s -> verhindern eines overflow
		} while (BUTTON_PUSHED);
		if (cT > 100) {                             // lnger 1 s: Speichern
          iFreq = Fm_tune_status();
          eeprom_update_word(pUint_EEaddr,iFreq);
          // 'Print #1 , F , N */
          pUint_EEaddr ++;		     				// Pointer-Inkrement
          iFreq = 0xFFFF;                           // Endemarke
          eeprom_update_word(pUint_EEaddr,iFreq);
		}; // end if
      } // end of program loop forever
	}; // end if
	
  // Anzahl gespeicherter Kanle auslesen 
  // cN = 0;
  cNmax = 0;
  pUint_EEaddr = 0;
  do {
    iFreq=eeprom_read_word(pUint_EEaddr);
	if (iFreq < 10800) cNmax++;
	pUint_EEaddr++;
  } while(iFreq != 0xFFFF);
  
  
  if (cNmax == 0) Scan();
  cN = cNmax * 2;
  cN--;
  iDiv = 1024 / cN;
 // cCh = 0;
  cCh_old = 255;
  for (;;) {						// loop forever
    iPoti = ADC_Read(2);			// ADC single conversion for ch ADC2 
    iPoti = iPoti / iDiv;
    if (!(CHECKBIT(iPoti,0))) {     // Check if Bit 0 of iPoti is cleared 
       cCh = iPoti / 2;				// -> Hyterese
    }; // end if
    if (cCh != cCh_old) {
      pUint_EEaddr = PUINTCAST(cCh); 					// orig: cCh*2 due to byte - word issue
      // Readeeprom F , N
	  iFreq=eeprom_read_word(pUint_EEaddr);
      si4735freq(iFreq);
      cCh_old = cCh;
    }; // end if
  } // end of loop forever


	return 0; // main return value
} // end of main

void si4735init(void)
{
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x01);                 // write value
    i2c_write(0x10);                 // write value
	i2c_write(0x05);                 // write value
    i2c_stop();
    _delay_ms(500);  
}

void si4735freq(unsigned int iF_)
{
	unsigned char cLow;
	unsigned char cHigh;

	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x20);                 // write value
    i2c_write(0x00);                 // write value
    cHigh = iF_ >> 8;			 // High Byte of iFreq
    cLow =  iF_ & 0xFF;			 // Low Byte of iFreq
	i2c_write(cHigh);
	i2c_write(cLow);
	i2c_write(0x00);                 // write value
    i2c_stop();
//    _delay_ms(500);  
}


void Scan(void)
{
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x21);                 // write value
    i2c_write(0x08);                 // write value
    i2c_stop();
//    _delay_ms(500);  
}

unsigned int Fm_tune_status(void)
{
  unsigned int iF_;
  unsigned char R1, R2, R3;

  UNUSED(R1);						// avoid compiler warning about unused var
  i2c_start(Dev4735+I2C_WRITE);		// set device address and write mode
  i2c_write(0x22);                 	// write value
  i2c_write(0x03);
  i2c_stop();
  i2c_start(Dev4735+I2C_READ);    	// set device address and read mode
  R1=i2c_readAck();
  R1=i2c_readAck();
  R2=i2c_readAck();
  R3=i2c_readAck();
  R1=i2c_readNak();
  iF_ = R2 << 8;				 	// High Byte creation
  iF_ += R3;						// Low byte add
  i2c_stop();
  return iF_;
}


void ADC_Init(void)
{

  // die Versorgungsspannung AVcc als Referenz whlen - default:
  // REFS0 = 0; REFS1 = 0

  // Bit ADFR ("free running") in ADCSRA steht beim Einschalten
  // schon auf 0, also single conversion

  ADCSRA = (1<<ADPS2) ;     // Frequenzvorteiler 1:16 fuer 50kHz < fadc < 200kHz
  ADCSRA |= (1<<ADEN);      // ADC aktivieren
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
 
  ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) ) {         // auf Abschluss der Konvertierung warten
  }
  /* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
     Wandlung nicht bernommen. */
  (void) ADCW;
	
}


/* ADC Einzelmessung */
unsigned int ADC_Read( unsigned char channel )
{
  // Kanal waehlen, ohne andere Bits zu beeinflussen
  ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
  ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
  while (ADCSRA & (1<<ADSC) );    // auf Abschluss der Konvertierung warten
  return ADCW;                    // ADC auslesen und zurckgeben
}




