//----------------------------------------------------------------------
// Titel     : DSP Heimradio mit ATTINY13A/85 fuer SI4735 Radio
//----------------------------------------------------------------------
// Funktion  : Misst Spannung via Poti und ermittelt Stufen zur
//				 fixen Frequenzstufenwahl
// Schaltung : siehe B. Kainka
// Dateiname : DSP_heimradio1.c
// Includes  : i2cmaster.h 
// Hinweis   : i2cmaster.S (Assem. Source) den source Files hinzuf.
//----------------------------------------------------------------------
// Prozessor : Attiny13A / Attiny85
// Takt       : 1.00 MHz (RC Oszillator 8MHz, CLKDIV8 aktiviert)
// Sprache   : C
// Datum     : 16.03.2014 11:59:33
// Version   : 1.06
// Autor     : Andreas Nestler
// Changes   : 
//		v1   : initial version; Si4735 is not commandable
//		v1.01: i2_start_wait statt i2c_start fr si4735init,
//             da das IC mehr Zeit braucht, bevor die folgenden Daten
//             geschrieben werden knnen. Die folgenden Zugriffe sind
//             nicht betroffen	
//      v1.02: MONO operation mode forced within si4735init routine
//      v1.03: MONO opMode removed,
//             calculation for channel selection changed N++ instead N--
//      v1.04: Property check with Volume control if button is toggled 
//             in OpMode
//      v1.05: FM_RSQ_STATUS added
//      v1.06: Some Properties added
//----------------------------------------------------------------------
// Pinconfig:
// =====================
// PB0 -> SDA	 (O)	-> TWI Daten
// PB1 -> Taste  (I) PU	-> Taste an GND
// PB2 -> SCL	 (O)	-> TWI Takt  
// PB3 -> unused (I) 	-> auf GND legen,
// PB4 -> ADC2   (AI) 	-> Potischleifer
// PB5 -> unused (I) PU -> reserviert als RESET-Pin
//
// Legende: I = Input , O = Output, AI = Analog Input, PU = Pull-up
//
//----------------------------------------------------------------------

#ifndef F_CPU
#define F_CPU 1000000UL
#endif

#include <inttypes.h>      // Allgemeine Bibliotheken
#include <avr/io.h>
#include <avr/interrupt.h>
// #include <avr/signal.h> // obsolete replaced by interrupt.h
#include <util/delay.h>
#include <avr/eeprom.h>
#include "inc/i2cmaster.h"
// #include "inc/USI_UART.h" // add USI_UART.c to source files

/* Hardware IO abstraction macros */

/* Button abstraction */
#define BUTTON_PUSHED (!(PINB & (1<<PINB1))) // !, da Taster an PB1 nach GND
/* Bit Manipulation */
#define SETBIT(VAR, BIT) ((VAR) |= (1 << (BIT))) // set bit
#define CLEARBIT(VAR, BIT) ((VAR) &= (unsigned)~(1 << (BIT))) // clear bit
#define TOGGLEBIT(VAR, BIT) ((VAR) ^= (1 << (BIT))) // Toogle Bit
#define CHECKBIT(VAR, BIT) ((VAR) & (1 << (BIT))) // Check if Bit is set

/* Defines to avoid compiler warnings */
#define UNUSED(x) ((void)(x))
// Doppel-Cast, um compiler warning zu vermeiden:
// 'cast to pointer from integer of different size'
#define PUINTCAST(x) ((unsigned int *)(uintptr_t) (x))


/*
** constant definitions
*/
#define Dev4735  0x22   // device address of SI4735 see datasheet
						// -> 34d  = 0x11 once left shifted
#define F1 8890			// Freq.: N-Joy
#define F2 9350			// Freq.: NDR2
#define F3 9730			// Freq.: Antenne-MV
#define F4 10480		// Freq.: Ostseewelle
#define F5 10650		// Freq.: DLF

/*
** type definitions
*/ 
typedef struct RSQStruct {   		/* deklariert den Strukturtyp RSQStruct */
  unsigned char stblend;
  unsigned char rssi;
  unsigned char snr;				
} tRSQ;							    /* name of new created type */

/*
** function prototypes
*/ 
void si4735init(void);
void fm_deemphasis(void);
void fm_max_tune_error(void);
void fm_seek_tune_snr_threshold(void);
void fm_seek_tune_rssi_threshold(void);
void fm_seek_freq_spacing(void);
void si4735freq(unsigned int iFreq);
void Scan(void);
unsigned int Fm_tune_status(void);
tRSQ Fm_rsq_status(void);
void ADC_Init(void);
unsigned int ADC_Read(unsigned char channel);
void Mono_Ctrl( unsigned char cButPush_Flag);
void GPO_set(unsigned char cSetVal);
void wait_for_ready_CTS(void);


/*
** Default Frequenzen inkl. Endemarke in EEPROM
** muss global definiert werden!
*/
int EEMEM Freq_list[6]={F1, F2, F3, F4, F5, 0xFFFF};

/*
** globale Variablen
*/
tRSQ rsq;							/* deklariert Variable des Typs tRSQ */
unsigned char ready=0;

int main(void)
{
    unsigned char cT, cN, cNmax, cCh=0, cCh_old, cButPush_Flag=0, cButPush_long_Flag=0; 
	unsigned int iFreq, iPoti, iDiv, * pUint_EEaddr;

		
        
	DDRB = (1<<DDB0) | (1<<DDB2);	// Portpins B.0 und B.2 auf Ausgang,
    PORTB = (1<<PB0) | (1<<PB2);	// und diese Bits auf 1
	PORTB |= (1<<PB1) | (1<<PB5);	// noch Pull-up f. Inputs PB1 u. PB5 ein
	DIDR0 = (1<<DDB4);				// Disable dig. input buffer for PB4->ADC2
									// this saves current consumption
	/* call init functions */
	ADC_Init();
	i2c_init();
	_delay_ms(500);
	si4735init();
	fm_deemphasis();				// set correct FM deemphasis of 50us
	
	
	/* endless loop for programming mode */
    if BUTTON_PUSHED {				// Programmier-Modus wenn Taste 
	  do {							// beim Einschalten gedrckt
	    _delay_ms(10);				// Tasten-Entprellung
	  } while (BUTTON_PUSHED);
	  iFreq=8700;
	  fm_seek_tune_snr_threshold(); // SNR threshold for seek
	 // fm_seek_tune_rssi_threshold(); // 0x14 -> default
	  fm_seek_freq_spacing();		// frequency spacing for seek
	  si4735freq(iFreq);			// Frequenz auf UKW-Bandanfang einstellen
	  _delay_ms(200);
	  pUint_EEaddr=0;				// setze Pointer auf 0 -> EEAddr 0
	  for (;;) { 					// program loop forever
	    Scan();						// Neuen kanal suchen
		do {
	      _delay_ms(10);
		} while (!BUTTON_PUSHED);
		cT=0;						// cT initialisieren
		do {
		  _delay_ms(10);
		  cT++;   									// Lnge des Tastenducks messen -> 10ms inkrement
		  if (cT > 200) cT=200;						// Limitieren auf 2s -> verhindern eines overflow
		} while (BUTTON_PUSHED);
		if (cT > 100) {                             // lnger 1 s: Speichern
          iFreq = Fm_tune_status();
          eeprom_update_word(pUint_EEaddr,iFreq);
          // 'Print #1 , F , N */
          pUint_EEaddr ++;		     				// Pointer-Inkrement
          iFreq = 0xFFFF;                           // Endemarke
          eeprom_update_word(pUint_EEaddr,iFreq);
		}; // end if
      } // end of program loop forever
	}; // end if BUTTON_PUSHED; programming mode
	
  // Start Opmode
  // Anzahl gespeicherter Kanle auslesen -> Nmax
  // cN = 0; // was in original BASCOM file, obsolete
  cNmax = 0;
  pUint_EEaddr = 0;
  do {
    iFreq=eeprom_read_word(pUint_EEaddr);
	if (iFreq < 10800) cNmax++;
	pUint_EEaddr++;
  } while(iFreq != 0xFFFF);

//  if (cNmax == 0) Scan();  // falls keine Sender gespeichert, einfach scannen
  cN = cNmax * 2;
  cN++;
  iDiv = 1024 / cN;
 // cCh = 0;
  cCh_old = 255;
  for (;;) {						// loop forever
    CLEARBIT(cButPush_long_Flag,0); // reset flag
	/* Im Opmode hat die Taste folgende Funktionen:
	-----------------------------------------------
	  Taste kurz gedrckt:
	   -> Mono/Loudness Control,
	   -> cButPush_Flag toggeln
	  Taste >1s gedrckt 
	   -> RSQ Daten lesen,
	   -> als Pulsbreite value x 1ms auf GPO1,
	   -> Daten in EEPROM ab Addr. 100 wegschreiben
	   -> cButPush_long_Flag setzen/lschen
	-----------------------------------------------   */
    if BUTTON_PUSHED {				// Tastenabfrage im OpMode
	  do {							// Test auf Taste gedrueckt
	    _delay_ms(10);				// Tasten-Entprellung
	  } while (BUTTON_PUSHED); 
      TOGGLEBIT(cButPush_Flag,0);
	  do {
		_delay_ms(10);
	  } while (!BUTTON_PUSHED);
	  cT=0;						    // cT initialisieren
	  do {
		_delay_ms(10);
		cT++;   									// Lnge des Tastenducks messen -> 10ms inkrement
		if (cT > 200) cT=200;						// Limitieren auf 2s -> verhindern eines overflow
	  } while (BUTTON_PUSHED);
	  if (cT > 100) {
	    SETBIT(cButPush_long_Flag,0);               // Flag setzen, wenn Tastendruck > 1s
	  }
	  else CLEARBIT(cButPush_long_Flag,0);          // andernfalls Flag zurcksetzen
	}; //end if 
	_delay_ms(1);
	 Mono_Ctrl(cButPush_Flag);		 // call MonoCtrl function
     iPoti = ADC_Read(2);			 // ADC single conversion for ch ADC2 
     iPoti = iPoti / iDiv;
     if (!(CHECKBIT(iPoti,0))) {     // Check if Bit 0 of iPoti is cleared 
        cCh = iPoti / 2;			 // -> Hyterese
     }; // end if
     if (cCh != cCh_old) {  // Neue Frequenz einstellen, wenn Kanal gendert wurde
       pUint_EEaddr = PUINTCAST(cCh);// orig: cCh*2 due to byte - word issue
       // Readeeprom F , N
       iFreq=eeprom_read_word(pUint_EEaddr);
       si4735freq(iFreq);
       cCh_old = cCh;
      }; // end if
	  if (cButPush_long_Flag) { /* RSQ Auswertung nur bei langem Tastendruck */
			rsq = Fm_rsq_status();
			GPO_set(0x02); // Output High
			for (cT=0;cT<=rsq.rssi;cT++)
			{
				_delay_ms(1);
			}
			GPO_set(0x00); //Output Low
			// Schreibe RSQ Daten ins EEPROM ab Addresse 100
			pUint_EEaddr=PUINTCAST(100);
			eeprom_update_word(pUint_EEaddr,rsq.rssi);
			pUint_EEaddr++;
			eeprom_update_word(pUint_EEaddr,rsq.snr);
			pUint_EEaddr++;
			eeprom_update_word(pUint_EEaddr,rsq.stblend);
			pUint_EEaddr++;
			eeprom_update_word(pUint_EEaddr,0xFFFF);
			pUint_EEaddr++;
			eeprom_update_word(pUint_EEaddr,cNmax);
			pUint_EEaddr++;
			eeprom_update_word(pUint_EEaddr,cCh);
			pUint_EEaddr++;
			eeprom_update_word(pUint_EEaddr,iPoti);
			
	  };	  
  } // end of loop forever
  return 0; // main return value
} // end of main

/* initialize the Si4735 */
void si4735init(void)
{
	i2c_start_wait(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x01);                 // Device power-up
    i2c_write(0x10);                 // FUNC, XOSCEN -> FM, 32kHz Oscillator on
	i2c_write(0x05);                 // OPMODE: Analog audio outputs
    i2c_stop();
	_delay_ms(100);					 // Pause von 100ms ntig, sonst ist GPO CTL setting nicht mglich
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
	i2c_write(0x80);                 // SET_PROPERTY
	i2c_write(0x02);                 // GPO1 as output
	i2c_stop();                 
    _delay_ms(500);  
}

/*---------------------------------------------------------------------------*/
/* Some property settings */
/*---------------------------------------------------------------------------*/

void fm_deemphasis(void)        //FM Deemphasis einstellen
{             
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x11);
   i2c_write(0x00);           
   i2c_write(0x00);    
   i2c_write(0x01);     //Deemphasis = 50s -> valid for european FM
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}
        
void fm_max_tune_error(void)       //FM Max Tune Error
{
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x11);
   i2c_write(0x08);           
   i2c_write(0x00);    
   i2c_write(0x14);     //max Tune Error 14=20kHz (default), 28=40KHz, 32=50KHz
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}

void fm_seek_tune_snr_threshold(void)        //FM_SEEK_TUNE_SNR_THRESHOLD
{      
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x14);
   i2c_write(0x03);           
   i2c_write(0x00);    
   i2c_write(0x06);     //FM_SEEK_TUNE_SNR_THRESHOLD 6dB (3dB def)
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}

void fm_seek_tune_rssi_threshold(void)       //FM_SEEK_TUNE_RSSI_THRESHOLD
{
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x14);
   i2c_write(0x04);           
   i2c_write(0x00);    
   i2c_write(0x14);     //FM_SEEK_TUNE_RSSI_THRESHOLD 20dBuV (def)
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}

void fm_seek_freq_spacing(void)        //FM_SEEK_FREQ_SPACING
{
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
	i2c_write(0x12);
	i2c_write(0x00);
	i2c_write(0x14);
	i2c_write(0x02);
	i2c_write(0x00);
	i2c_write(0x05);     //FM_SEEK_FREQ_SPACING 50 KHz, default is 0x0A -> 100kHz
	i2c_stop();
	wait_for_ready_CTS();
	_delay_ms(1);
}

/* Set tuning frequency */
void si4735freq(unsigned int iF_)
{
	unsigned char cLow;
	unsigned char cHigh;

	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x20);                 // write value
    i2c_write(0x00);                 // write value
    cHigh = iF_ >> 8;			 	 // High Byte of iFreq
    cLow =  iF_ & 0xFF;			 	 // Low Byte of iFreq
	i2c_write(cHigh);
	i2c_write(cLow);
	i2c_write(0x00);                 // write value
    i2c_stop();
	wait_for_ready_CTS();			 // wait for clear to send
//    _delay_ms(500);  
}


void Scan(void)
{
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x21);                 // FM_SEEK_START
    i2c_write(0x08);                 // SEEK UP , for Wrap function write 0x0C
    i2c_stop();
//    _delay_ms(500);  
	_delay_ms(60);
	wait_for_ready_CTS();
}

/* Read Frequency from Fm_Tune_Status function */
unsigned int Fm_tune_status(void)
{
  unsigned int iF_;
  unsigned char R1, R2, R3;

  UNUSED(R1);						// avoid compiler warning about unused var
  i2c_start(Dev4735+I2C_WRITE);		// set device address and write mode
  i2c_write(0x22);                 	// command Fm_Tune_status
  i2c_write(0x03);					// command argument 1
  i2c_stop();
  i2c_start(Dev4735+I2C_READ);    	// set device address and read mode
  R1=i2c_readAck();					// dummy read => STATUS
  R1=i2c_readAck();					// Response 1
  R2=i2c_readAck();					// Response 2 -> Freq H
  R3=i2c_readAck();					// Response 3 -> Freq L
  R1=i2c_readNak();					// Response 4 -> RSSI
									// Response 5 - 7 not read
  iF_ = R2 << 8;				 	// High Byte creation
  iF_ += R3;						// Low byte add
  i2c_stop();
  return iF_;
}

/* Read out receive signal quality */
tRSQ Fm_rsq_status(void)
{
  tRSQ rsq_;
  unsigned char R1;

  UNUSED(R1);						// avoid compiler warning about unused var
  i2c_start(Dev4735+I2C_WRITE);		// set device address and write mode
  i2c_write(0x23);                 	// write value for fm_rsq_status
  i2c_write(0x01);					// INT status cleared
  i2c_stop();
  wait_for_ready_CTS();				// wait for clear to send -> needed!
  i2c_start(Dev4735+I2C_READ);    	// set device address and read mode
  R1=i2c_readAck();					// dummy read => STATUS
  R1=i2c_readAck();					// Response 1
  R1=i2c_readAck();					// Response 2
  rsq_.stblend=i2c_readAck();		// Response 3 -> PILOT + STBLEND
  rsq_.rssi=i2c_readAck();			// Response 4 -> RSSI
  rsq_.snr=i2c_readAck();			// Response 5 -> SNR
  R1=i2c_readAck();					// Response 6
  R1=i2c_readNak();					// Response 7
  i2c_stop();
  return rsq_;
}


/* Initialize the ADC */
void ADC_Init(void)
{

  // die Versorgungsspannung AVcc als Referenz whlen - default:
  // REFS0 = 0; REFS1 = 0

  // Bit ADFR ("free running") in ADCSRA steht beim Einschalten
  // schon auf 0, also single conversion

  ADCSRA = (1<<ADPS2) ;     // Frequenzvorteiler 1:16 fuer 50kHz < fadc < 200kHz
  ADCSRA |= (1<<ADEN);      // ADC aktivieren
 
  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
 
  ADCSRA |= (1<<ADSC);                  // eine ADC-Wandlung 
  while (ADCSRA & (1<<ADSC) ) {         // auf Abschluss der Konvertierung warten
  }
  /* ADCW muss einmal gelesen werden, sonst wird Ergebnis der nchsten
     Wandlung nicht bernommen. */
  (void) ADCW;
	
}


/* ADC Einzelmessung from channel */
unsigned int ADC_Read( unsigned char channel )
{
  // Kanal waehlen, ohne andere Bits zu beeinflussen
  ADMUX = (ADMUX & ~(0x1F)) | (channel & 0x1F);
  ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
  while (ADCSRA & (1<<ADSC) );    // auf Abschluss der Konvertierung warten
  return ADCW;                    // ADC auslesen und zurckgeben
}

/* Loudness/Mono control */
void Mono_Ctrl( unsigned char cBP_Flag)
{
  unsigned char cVol, cMono_Blend_Th;
  
  if (cBP_Flag) {
    cVol = 0x3C; // MONO ein Tick leiser
	cMono_Blend_Th = 0x7F; // 127 -> MONO forced
  }
  else {
    cVol = 0x3F;
	cMono_Blend_Th = 0x00; // 0 -> Stereo forced; 30uV -0x1E Threshold - default
  };
  i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
  i2c_write(0x12);                 // SET_PROPERTY
  i2c_write(0x00);				   // ARG 1 always 0 for properties
  i2c_write(0x40);                 // High byte of 0x1106 FM_BLEND_MONO
  i2c_write(0x00);                 // Low byte of 0x1106 FM_BLEND_MONO
  i2c_write(0x00);                 // High byte of property value -> 0x00
  i2c_write(cVol);                 // Low byte of property value -> 7F / 1E
  i2c_stop();
  wait_for_ready_CTS();
  /*Set into Mono mode */
  i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
  i2c_write(0x12);                 // SET_PROPERTY
  i2c_write(0x00);				   // ARG 1 always 0 for properties
  i2c_write(0x11);                 // High byte of 0x1106 FM_BLEND_MONO
  i2c_write(0x06);                 // Low byte of 0x1106 FM_BLEND_MONO
  i2c_write(0x00);                 // High byte of property value -> 0x00
  i2c_write(cMono_Blend_Th);                 // Low byte of property value -> 7F / 1E
  i2c_stop(); 
  wait_for_ready_CTS();
}

void GPO_set(unsigned char cSetVal)
{
 	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
	i2c_write(0x81);                 // SET_PROPERTY
	i2c_write(cSetVal);              // set output value of GPOs
	i2c_stop();     
}

/* Clear to send function */
void wait_for_ready_CTS(void)
{
	while(!(ready & 0x80))
	{
		i2c_start(Dev4735+I2C_READ);
		ready=i2c_readNak();
		i2c_stop();
	}
	ready=0;
}



